import 'package:dynamic_widget/dynamic_widget.dart';
import 'package:flutter/material.dart';
import 'package:flutter_form_builder_ing/app_colors.dart';
import 'package:flutter_form_builder_ing/flutter_form_builder_ing.dart';
import 'package:intl/intl.dart';
import 'package:layout_form_builder/datalayer/bean/eform_cache.dart';
import 'package:layout_form_builder/datalayer/constant/eform_properties.dart';
import 'package:line_awesome_flutter/line_awesome_flutter.dart';


class DatePicker extends StatefulWidget {

  static const PROP = [
    EFormProperties.PROP_INPUT_ID,
    EFormProperties.PROP_COMPULSORY,
    EFormProperties.PROP_ON_LOAD_TRIGGERS,
    EFormProperties.PROP_ON_END_EDITING_TRIGGERS,
    EFormProperties.PROP_COMPULSORY_ERR_MSG,
    EFormProperties.PROP_INPUT_LABEL,
    EFormProperties.PROP_INPUT_HINT,
    EFormProperties.PROP_MIN_DATE,
    EFormProperties.PROP_MAX_DATE,
    EFormProperties.PROP_MIN_DATE_ERR_MSG,
    EFormProperties.PROP_MAX_DATE_ERR_MSG,
    EFormProperties.PROP_INPUT_ENABLED,
    EFormProperties.PROP_COLLABORATION_GROUP_ID,
    EFormProperties.PROP_DEFAULT_VALUE,
    EFormProperties.PROP_INPUT_VISIBLE,
    EFormProperties.PROP_VISIBLE_ON_PDF,
  ];

  final Map<String, dynamic> fieldProp;
  final ClickListener? listener;

  DatePicker({Key? key, required this.fieldProp, required this.listener}) : super(key: key);
  
  @override
  _DatePickerState createState() => _DatePickerState();
}

class _DatePickerState extends State<DatePicker> {

  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> fp = widget.fieldProp;
    bool enabled = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_INPUT_ENABLED, defaultValue: true);
    String? label = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_LABEL);
    String? hint = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_HINT);
    DateTime? minDate = EFormCache.getFormFieldPropDateTime(context, fp, EFormProperties.PROP_MIN_DATE);
    DateTime? maxDate = EFormCache.getFormFieldPropDateTime(context, fp, EFormProperties.PROP_MIN_DATE);

    final textStyle = EFormCache.getTextStyle();
    final inputDecoration = EFormCache.getInputDecoration(
        label: label ?? "",
        hint: hint ?? "Select date",
        suffix: Icon(LineAwesomeIcons.calendar, color: AppColors.TextGray, size: 22),
    );

    return Theme(
        data: new ThemeData(
          primaryColor: AppColors.TextGray,
          primaryColorDark: AppColors.TextGray,
        ),
        child: Padding(
          padding: EdgeInsets.all(5),
          child: FormBuilderDateTimePicker(
            inputType: InputType.date,
            name: fp[EFormProperties.PROP_INPUT_ID],
            firstDate: minDate,
            lastDate: maxDate,
            style: textStyle?.textStyle ?? TextStyle(fontSize: 14.0, fontFamily: 'Lato', color: Colors.black),
            validator: (val) {
              try {
                return _validate(val);
              }catch(e, s){
                print("DateRangePicker: Failed to validate with error: $e, stacktrace: $s");
              }
              return null;
            },
            enabled: enabled,
            autovalidateMode: AutovalidateMode.disabled,
            decoration: inputDecoration,
          ),
        )
    );
  }

  String? _validate(dynamic value) {
    // String? val = DateFormat("dd/MM/yyyy").format(value);
    Map<String, dynamic> fp = widget.fieldProp;
    bool? compulsory = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_COMPULSORY, fromValidator: true, defaultValue: false);
    
    if (compulsory && value == null) {
      // if (val == null || val.length == 0) {
        return EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_COMPULSORY_ERR_MSG, fromValidator: true);
      // }
    }
    return null;
  }
}